diff a/src/jdk.compiler/share/classes/com/sun/tools/javac/tree/JCTree.java b/src/jdk.compiler/share/classes/com/sun/tools/javac/tree/JCTree.java
--- a/src/jdk.compiler/share/classes/com/sun/tools/javac/tree/JCTree.java
+++ b/src/jdk.compiler/share/classes/com/sun/tools/javac/tree/JCTree.java
@@ -36,10 +36,11 @@
 import com.sun.source.tree.*;
 import com.sun.tools.javac.code.*;
 import com.sun.tools.javac.code.Directive.RequiresDirective;
 import com.sun.tools.javac.code.Scope.*;
 import com.sun.tools.javac.code.Symbol.*;
+import com.sun.tools.javac.code.Type.MethodType;
 import com.sun.tools.javac.util.*;
 import com.sun.tools.javac.util.DefinedBy.Api;
 import com.sun.tools.javac.util.JCDiagnostic.DiagnosticPosition;
 import com.sun.tools.javac.util.List;
 
@@ -751,23 +752,35 @@
     }
 
     /**
      * Common supertype for all functional expression trees (lambda and method references)
      */
-    public abstract static class JCFunctionalExpression extends JCPolyExpression {
+    public abstract static sealed class JCFunctionalExpression extends JCPolyExpression
+                                                               permits JCLambda, JCMemberReference {
 
         public JCFunctionalExpression() {
             //a functional expression is always a 'true' poly
             polyKind = PolyKind.POLY;
         }
 
         /** list of target types inferred for this functional expression. */
         public Type target;
+        /** code reflection specific metadata. */
+        public CodeReflectionInfo codeReflectionInfo;
 
         public Type getDescriptorType(Types types) {
-            return target != null ? types.findDescriptorType(target) : types.createErrorType(null);
+            if (target == null) {
+                return types.createErrorType(null);
+            } else if (target.hasTag(TypeTag.METHOD)) {
+                // this is a quoted expression
+                return target;
+            } else {
+                return types.findDescriptorType(target);
+            }
         }
+
+        public record CodeReflectionInfo(Symbol quotedField, List<JCExpression> capturedArgs) { }
     }
 
     /**
      * A class definition.
      */
@@ -1955,11 +1968,11 @@
     }
 
     /**
      * A lambda expression.
      */
-    public static class JCLambda extends JCFunctionalExpression implements LambdaExpressionTree {
+    public static final class JCLambda extends JCFunctionalExpression implements LambdaExpressionTree {
 
         public enum ParameterKind {
             IMPLICIT,
             EXPLICIT
         }
@@ -2541,11 +2554,11 @@
     }
 
     /**
      * Selects a member expression.
      */
-    public static class JCMemberReference extends JCFunctionalExpression implements MemberReferenceTree {
+    public static non-sealed class JCMemberReference extends JCFunctionalExpression implements MemberReferenceTree {
 
         public ReferenceMode mode;
         public ReferenceKind kind;
         public Name name;
         public JCExpression expr;
